<?php
/*
Plugin Name: Simple WP Retina
Plugin URI: http://wordpress.org/extend/plugins/simple-wp-retina/
Description: Adds seamless support for Retina and other high pixel density screens.
Version: 1.0
Author: Jonathan Desrosiers & Slocum Design Studio
Author URI: http://slocumstudio.com
License: GPLv2
*/

class Simple_WP_Retina {

	function __construct() {
		add_action( 'after_setup_theme', array( $this, 'after_setup_theme' ), 999 );
		add_action( 'wp_footer', array( $this, 'wp_footer' ) );
		add_filter( 'the_content', array( $this, 'the_content' ) );
		add_filter( 'post_thumbnail_size', array( $this, 'post_thumbnail_size' ), 999, 1 );
		add_filter( 'post_thumbnail_html', array( $this, 'post_thumbnail_html' ), 999, 5 );
	}

	//Adds an @2x image size for core and theme defined image sizes
	function after_setup_theme() {

		$image_sizes = get_intermediate_image_sizes();
		foreach( $image_sizes as $image_size ) {
			if ( $image = $this->get_image_size( $image_size ) )
				add_image_size( $image_size . '@2x', $image['width'] * 2, $image['height'] * 2, $image['crop'] );
			else if ( $image_size == 'thumb' || $image_size == 'thumbnail' )
				add_image_size( 'thumbnail@2x', intval( get_option( 'thumbnail_size_w' ) ) * 2, intval( get_option( 'thumbnail_size_h' ) ) * 2, true );
			else if ( $image_size == 'medium' )
				add_image_size( 'medium@2x', intval( get_option( 'medium_size_w' ) ) * 2, intval( get_option( 'medium_size_h' ) ) * 2, false );
			else if ( $image_size == 'large' )
				add_image_size( 'large@2x', intval( get_option( 'large_size_w' ) ) * 2, intval( get_option( 'large_size_h' ) ) * 2, false );
		}
	}

	//makes surethat a given image name is a registered image size
	function get_image_size( $name ) {
		global $_wp_additional_image_sizes;

		if ( isset( $_wp_additional_image_sizes[ $name ] ) )
			return $_wp_additional_image_sizes[ $name ];

		return false;
	}

	//verifies that the user's screen is a high pixel density display
	function is_high_res() {
		if ( isset( $_COOKIE['devicePixelRatio'] ) && $_COOKIE['devicePixelRatio'] > 1.5 )
			return true;
		else
			return false;
	}

	//Changes the post thumbnail size to @2x
	function post_thumbnail_size( $size ) {
		if ( $this->is_high_res() )
			$size .= '@2x';
		return $size;
	}

	function post_thumbnail_html( $html, $post_id, $post_thumbnail_id, $size, $attr ) {
		if ( $this->is_high_res() ) {
			//find images
		    preg_match_all( "#<img(.*?)\/?>#", $html, $matches );

		    $new_images = array();

		    foreach ( $this->break_apart_images( $matches ) as $img_key => $image ) {
		    	$new_images[ $img_key ] = array( $matches[0][ $img_key ] );

			    $image['width'] = (int) ( $image['width'] / 2 );
			    $image['height'] = (int) ( $image['height'] / 2 );
			    $image['class'] .= ' attachment-' . str_replace( '@2x', '', $size );

			    $new_html = '<img';

			    foreach ( $image as $attr_name => $attr_value )
				    $new_html .= ' ' . $attr_name . '="' . $attr_value . '"';
				$new_html .= ' />';

				$new_images[ $img_key ][] = $new_html;
		    }

		    foreach ( $new_images as $new )
		    	$html = str_replace( $new[0], $new[1], $html );
		}
	    return $html;
	}

	function break_apart_images( $images ) {
		 // extract attributes from each image and place in $images array
	    $image_attr = array();
	    foreach ( $images[1] as $img ) {
	        preg_match_all( "#(\w+)=['\"]{1}([^'\"]*)#", $img, $matches2 );
	        // code below could be a lot neater using array_combine(), but it's php5 only
	        $tempArray = array();
	        foreach( $matches2[1] as $key => $val )
	            $tempArray[ $val ] = $matches2[2][ $key ];

	        $image_attr[] = $tempArray;
	    }

	    return $image_attr;
	}

	function the_content( $content ) {
		$new_content = $content;

		if ( $this->is_high_res() ) {
			//find images
		    preg_match_all( "#<img(.*?)\/?>#", $content, $matches );

		    $defaults = array( 'size-medium', 'size-large', 'size-thumbnail' );

		    $new_image_tags = array();

		    foreach ( $this->break_apart_images( $matches ) as $key => $image ) {

			    $classes = explode( ' ', $image['class'] );

			    foreach ( $classes as $class_key => $value ) {

				    if ( in_array( $value, $defaults ) ) {
					    $classes[] .= $classes[ $class_key ] . '@2x';
					    $twox_class = substr( $classes[ $class_key ] . '@2x', 5 );
					}

				    if ( strstr( $value, 'wp-image-' ) ) {
					    $id_classes = explode( '-', $value );
					    $image_id = $id_classes[2];
				    }
			    }

			    if ( ! empty( $image_id ) && ! empty( $twox_class ) ) {

				    $new_src = wp_get_attachment_image_src( $image_id, $twox_class );

    			    $new_image_tags[ $key ] = array( $matches[0][ $key ] );

    			    $new_image_tags[ $key ][] = '<img src="' . $new_src[0] . '" width="' . $image['width'] . '" height="' . $image['height'] . '" class="' . implode( ' ', $classes ) . '" />';

				}
		    }

		    //Replace old image with new
		    foreach ( $new_image_tags as $new_img )
			    $new_content = str_replace( $new_img[0], $new_img[1], $new_content );
		}
		return $new_content;
	}

	function wp_footer() {
	?>
		<script type="text/javascript">// <![CDATA[
			if( window.devicePixelRatio !== undefined ) document.cookie = 'devicePixelRatio = ' + window.devicePixelRatio;
		// ]]></script>
		<?php
	}
}

$simple_wp_retina = new Simple_WP_Retina();